"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const fs_extra_1 = require("fs-extra");
const AsyncTaskQueue_1 = require("./AsyncTaskQueue");
const https_1 = require("https");
const url_1 = require("url");
const ProgressBar = require('progress');
const chalk = require('chalk');
const log = console.log;
class TinyPng {
    constructor(maxTasks = 100) {
        this.pendingList = [];
        this.successList = [];
        this.failedList = [];
        this.asyncTaskQueue = new AsyncTaskQueue_1.default(maxTasks);
    }
    setPendingList(list) {
        this.pendingList = list;
    }
    async run() {
        this.printStart();
        const bar = new ProgressBar('compressing [:bar] :percent :current/:total :etas left', {
            complete: chalk.green('#'),
            incomplete: ' ',
            width: 20,
            total: this.pendingList.length,
        });
        return new Promise((resolve) => {
            const taskFnQueue = this.pendingList.map((item) => async () => {
                try {
                    const res = await this.compressSinglePic(item.path, item.distPath);
                    this.successList.push({
                        ...item,
                        compressedSize: res.output.size,
                    });
                    bar.tick();
                }
                catch (err) {
                    bar.tick();
                    const errMsg = typeof err === 'string' ? err : (err || {}).msg;
                    this.failedList.push({ ...item, errMsg });
                }
            });
            this.asyncTaskQueue.setAsyncFnTasks(taskFnQueue);
            this.asyncTaskQueue.setFinishCallback(() => {
                this.printResult();
                if (this.failedList.length > 0) {
                    this.printFailed();
                }
                resolve(void 0);
            });
            this.asyncTaskQueue.run();
        });
    }
    /**
     * compress a image file and save it
     * @param sourcePath source image file path
     * @param distpath compressed image file dist path
     * @returns distpath
     */
    async compressSinglePic(sourcePath, distpath) {
        const res = await this.uploadSinglePic(sourcePath);
        const imgUrl = res.output.url;
        if (imgUrl) {
            await this.downloadSingleImg(imgUrl, distpath);
            return res;
        }
    }
    /**
     * upload a image
     * @param path source image path
     * @returns promise with upload result
     */
    async uploadSinglePic(path) {
        return new Promise((resolve, reject) => {
            fs_extra_1.readFile(path, (err, file) => {
                if (err) {
                    log('err', err);
                    reject(err);
                }
                const options = utils_1.randomRequestOption();
                const req = https_1.request(options, (res) => {
                    res.on('data', (data) => {
                        const obj = JSON.parse(data.toString());
                        obj.error ? reject(obj.message) : resolve(obj);
                    });
                });
                req.write(file, 'binary');
                req.on('error', (err) => {
                    reject(err);
                });
                req.end();
            });
        });
    }
    /**
     * download image from tiny.png
     * @param address image url to download
     * @param distPath the path image to be saved
     * @returns distPath
     */
    downloadSingleImg(address, distPath) {
        const opts = new url_1.URL(address);
        fs_extra_1.ensureFile(distPath).then(() => {
            const write = fs_extra_1.createWriteStream(distPath, {
                encoding: 'binary',
                flags: 'w',
            });
            return new Promise((resolve, reject) => {
                const req = https_1.request(opts, (res) => {
                    res.setEncoding('binary');
                    res.pipe(write, { end: false });
                    res.on('end', () => {
                        utils_1.markImageBeCompressed(write);
                        resolve(distPath);
                    });
                });
                req.on('error', (error) => reject(error));
                req.end();
            });
        });
    }
    printStart() {
        log(chalk.green `
Total File: ${this.pendingList.length}
    `);
    }
    /**
     * print the result, include originTotalSize, compressedTotalSize, compressRatio
     */
    printResult() {
        const originTotalSize = this.successList.reduce((prev, item) => prev + item.originSize, 0);
        const compressedTotalSize = this.successList.reduce((prev, item) => prev + item.compressedSize, 0);
        const compressRatio = (((originTotalSize - compressedTotalSize) * 100) /
            originTotalSize).toFixed(2);
        if (originTotalSize > 0) {
            const text = chalk `
SUCCESS: {green ${this.successList.length}}
FAILED: {red ${this.failedList.length}}
COMPRESS_RATIO: {yellow ${compressRatio}%}
      `;
            log(text);
        }
        else {
            log(chalk.red(`
No compressed images!`));
        }
    }
    printFailed() {
        const text = this.failedList
            .map((item) => {
            return chalk `{red  ${item.path}}: {red  ${item.errMsg}}
`;
        })
            .join('');
        log(chalk.yellow `
Failed File: ${this.failedList.length}
    `);
        log(text);
    }
}
exports.default = TinyPng;
