"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.judgeImageIsCompressed = exports.markImageBeCompressed = exports.getFullPath = exports.findAllImageFile = exports.canHandledImage = exports.randomRequestOption = exports.randomHeader = void 0;
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const TINYIMG_URL = ['tinyjpg.com', 'tinypng.com'];
/**
 * generate random X-Forward-For to avoid  to be forbidden by tinypng.com
 * @returns random headers
 */
const randomHeader = () => {
    const ip = new Array(4)
        .fill(0)
        .map(() => Math.round(Math.random() * 255))
        .join('.');
    return {
        'Cache-Control': 'no-cache',
        'Content-Type': 'application/x-www-form-urlencoded',
        authority: 'tinypng.com',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.116 Safari/537.36',
        'X-Forwarded-For': ip,
    };
};
exports.randomHeader = randomHeader;
/**
 * random request tinyjpg.com or tinypng.com
 * @returns random http request option
 */
const randomRequestOption = () => {
    const index = Math.random() < 0.5 ? 0 : 1;
    const headers = exports.randomHeader();
    return {
        headers,
        hostname: TINYIMG_URL[index],
        method: 'POST',
        path: '/backend/opt/shrink',
        rejectUnauthorized: false,
    };
};
exports.randomRequestOption = randomRequestOption;
const canHandledImage = (fileName) => {
    if (!fileName) {
        return false;
    }
    const name = fileName.toLowerCase();
    return ['.png', '.jpg', 'jpeg', '.webp'].some((ext) => name.endsWith(ext));
};
exports.canHandledImage = canHandledImage;
/**
 *
 * @param sourcePath complete path
 * @returns
 */
const findAllImageFile = (sourcePath) => {
    const toCompressList = [];
    let depth = 0;
    return new Promise((resolve, reject) => {
        (function traversePath(path) {
            depth++;
            fs_extra_1.readdir(path, (err, fileNames) => {
                depth--;
                if (err) {
                    reject(err);
                }
                fileNames &&
                    fileNames.forEach((fileName) => {
                        const filePath = path_1.join(path, fileName);
                        depth++;
                        fs_extra_1.stat(filePath, (err, stats) => {
                            depth--;
                            if (err) {
                                reject(err);
                            }
                            if (stats.isDirectory()) {
                                traversePath(filePath);
                            }
                            if (stats.isFile()) {
                                if (exports.canHandledImage(fileName)) {
                                    toCompressList.push({
                                        path: filePath,
                                        originSize: stats.size,
                                    });
                                }
                            }
                            if (depth === 0) {
                                resolve(toCompressList);
                            }
                        });
                    });
            });
        })(sourcePath);
    });
};
exports.findAllImageFile = findAllImageFile;
/**
 * get full path from relative path
 * @param path relative path
 * @returns full path
 */
const getFullPath = (path) => {
    return path_1.resolve(process.cwd(), path);
};
exports.getFullPath = getFullPath;
const MARK_TAG = 'tiny';
const markImageBeCompressed = (writeStream) => {
    writeStream.end(Buffer.from(MARK_TAG));
};
exports.markImageBeCompressed = markImageBeCompressed;
const judgeImageIsCompressed = (filePath) => {
    return new Promise((resolve) => {
        fs_extra_1.readFile(filePath, (err, buffer) => {
            if (err) {
                resolve(false);
                return;
            }
            const markStr = buffer.toString('ascii', buffer.length - MARK_TAG.length, buffer.length);
            resolve(markStr === MARK_TAG);
        });
    });
};
exports.judgeImageIsCompressed = judgeImageIsCompressed;
